@@ -17,7 +17,7 @@ from zebrazoom.dataAnalysis.datasetcreation.getTailAngleRecalculated2 import get
 from zebrazoom.dataAnalysis.datasetcreation.gatherInitialRawData import gatherInitialRawData
 import pickle
 
-def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecreation=0, addToGlobalParameters=0):
+def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecreation=0, addToGlobalParameters=0, minimumFrameToFrameDistanceToBeConsideredAsMoving=0, supstructOverwrite={}):
 
   # Gathering user-inputed information about how to create the dataframe of parameters for the whole set of videos
   
@@ -56,7 +56,7 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   else:
     print("You must provide either an excel file or a video name to create a dataframe of parameters.")
   
-  if nbFramesTakenIntoAccount == -1:
+  if nbFramesTakenIntoAccount <= -1:
     boutNbFrames = []
     boutTakenIntoAcccount = 0
     videoId = 0
@@ -65,11 +65,12 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
     else:
       path    = os.path.join(excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id'])
     trial_id  = excelFile.loc[videoId, 'trial_id']
-    include   = excelFile.loc[videoId, 'include']
-    include = eval('[' + include + ']')
-    include = include[0]
-    with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
-      supstruct = json.load(f)
+    include   = [bool(int(val.strip())) for val in excelFile.loc[videoId, 'include'][1:-1].split(',')]
+    if len(supstructOverwrite):
+      supstruct = supstructOverwrite
+    else:
+      with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
+        supstruct = json.load(f)
     for Well_ID, Cond in enumerate(include):
       if include[Well_ID]:
         for NumBout, dataForBout in enumerate(supstruct["wellPoissMouv"][Well_ID][0]):
@@ -79,23 +80,31 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
             break;
       if boutTakenIntoAcccount > 100:
         break;
-    nbFramesTakenIntoAccount = int(np.median(boutNbFrames))
+    if len(boutNbFrames):
+      if nbFramesTakenIntoAccount == -1:
+        nbFramesTakenIntoAccount = int(np.median(boutNbFrames))
+      else:
+        nbFramesTakenIntoAccount = 3 * int(np.median(boutNbFrames)) if 3 * int(np.median(boutNbFrames)) >= 100 else 100
+    else:
+      # There are not bouts in the dataset...
+      return [[], [], 0, []]
   
   # Creating labels of columns of dataframe
   # General basic information
   basicInformation = ['Trial_ID', 'Well_ID', 'NumBout', 'BoutStart', 'BoutEnd', 'Condition', 'Genotype', 'videoDuration']
   # Global parameters
   if tailAngleKinematicParameterCalculation:
-    globParam  = ['BoutDuration', 'TotalDistance', 'Speed', 'maxOfInstantaneousTBF', 'meanOfInstantaneousTBF', 'medianOfInstantaneousTBF', 'maxBendAmplitude', 'meanBendAmplitude', 'medianBendAmplitude', 'NumberOfOscillations', 'meanTBF', 'maxTailAngleAmplitude', 'deltaHead', 'firstBendTime', 'firstBendAmplitude', 'IBI', 'xmean', 'ymean', 'binaryClass25degMaxTailAngle']
-    if type(addToGlobalParameters) == list:
-      globParam = globParam + addToGlobalParameters
+    globParam  = ['BoutDuration', 'TotalDistance', 'Speed', 'maxOfInstantaneousTBF', 'meanOfInstantaneousTBF', 'medianOfInstantaneousTBF', 'maxBendAmplitude', 'maxBendAmplitudeSigned', 'meanBendAmplitude', 'medianBendAmplitude', 'medianBendAmplitudeSigned', 'NumberOfOscillations', 'meanTBF', 'maxTailAngleAmplitude', 'deltaHead', 'firstBendTime', 'firstBendAmplitude', 'firstBendAmplitudeSigned', 'IBI', 'xmean', 'ymean', 'binaryClass25degMaxTailAngle', 'tailAngleIntegralSigned', 'BoutFrameNumberStart', 'tailAngleSymmetry', 'secondBendAmpDividedByFirst', 'tailAngleIntegral', 'maxInstantaneousSpeed']
   else:
     globParam  = ['BoutDuration', 'TotalDistance', 'Speed']
   
+  if type(addToGlobalParameters) == list:
+    globParam = globParam + addToGlobalParameters  
+  
   # Initial raw data
   if saveRawDataInAllBoutsSuperStructure:
     if tailAngleKinematicParameterCalculation:
-      rawData = ['HeadX', 'HeadY', 'Heading', 'TailAngle_Raw', 'TailAngle_smoothed', 'Bend_Timing', 'Bend_TimingAbsolute', 'Bend_Amplitude', 'TailBeatFrequency']
+      rawData = ['HeadX', 'HeadY', 'Heading', 'TailAngle_Raw', 'TailAngle_smoothed', 'Bend_Timing', 'Bend_TimingAbsolute', 'Bend_Amplitude', 'TailBeatFrequency', 'curvature']
     else:
       rawData = ['HeadX', 'HeadY', 'Heading']
   # Tail angle related parameters for clustering
@@ -138,6 +147,7 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   removeColumnsWhenAppropriate = [col for col in dfCols if not(col in onlyKeepTheseColumns)]
   # Going through each video listed in the excel file
   for videoId in range(0, len(excelFile)):
+    
     if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder":
       path    = os.path.join(defaultZZoutputFolderPath, excelFile.loc[videoId, 'trial_id'])
     else:
@@ -145,23 +155,31 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
     trial_id  = excelFile.loc[videoId, 'trial_id']
     fq        = excelFile.loc[videoId, 'fq']
     pixelsize = excelFile.loc[videoId, 'pixelsize']
-    condition = excelFile.loc[videoId, 'condition']
-    condition = eval('[' + condition + ']')
-    condition = condition[0]
-    genotype  = excelFile.loc[videoId, 'genotype']
-    genotype = eval('[' + genotype + ']')
-    genotype = genotype[0]
-    include   = excelFile.loc[videoId, 'include']
-    include = eval('[' + include + ']')
-    include = include[0]
+    condition = [val.strip('\'" ') for val in excelFile.loc[videoId, 'condition'][1:-1].split(',')]
+    genotype  = [val.strip('\'" ') for val in excelFile.loc[videoId, 'genotype'][1:-1].split(',')]
+    include   = [bool(int(val.strip())) for val in excelFile.loc[videoId, 'include'][1:-1].split(',')]
+    
     if (not(os.path.exists(os.path.join(path, trial_id + '.pkl'))) or forcePandasDfRecreation):
-      with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
-        supstruct = json.load(f)
-        firstFrame = supstruct["firstFrame"]
-        lastFrame  = supstruct["lastFrame"]
+      if len(supstructOverwrite):
+        supstruct = supstructOverwrite
+      else:
+        with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
+          supstruct = json.load(f)
+      firstFrame = supstruct["firstFrame"]
+      lastFrame  = supstruct["lastFrame"]
+      with open(os.path.join(path, 'parametersUsedForCalculation.json'), 'w') as outfile:
+        print('frameStepForDistanceCalculation', frameStepForDistanceCalculation)
+        print('videoFPS', excelFile.loc[videoId, 'fq'])
+        print('videoPixelSize', excelFile.loc[videoId, 'pixelsize'])
+        json.dump({'frameStepForDistanceCalculation': int(frameStepForDistanceCalculation), 'videoFPS': float(excelFile.loc[videoId, 'fq']), 'videoPixelSize': float(excelFile.loc[videoId, 'pixelsize'])}, outfile)
     else:
       print("reloading previously calculated parameters")
       dfReloadedVid = pd.read_pickle(os.path.join(path, trial_id + '.pkl'))
+      
+      nbFramesTakenIntoAccountReloaded = max([np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()]), np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])])
+      if nbFramesTakenIntoAccountReloaded < nbFramesTakenIntoAccount:
+        raise ValueError("nbFramesTakenIntoAccount was too low when pre-generating the pkl file of a video:" + str(np.sum(['instaTBF' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['tailAnglesRecalculated' in param for param in dfReloadedVid.columns.tolist()])) + " , " + str(np.sum(['instaSpeed' in param for param in dfReloadedVid.columns.tolist()])) + " ; nbFramesTakenIntoAccount :" + str(nbFramesTakenIntoAccount))
+        
       for idx, cond in enumerate(condition):
         indForWellId = (dfReloadedVid['Well_ID'] == idx)
         if include[idx]:
@@ -175,7 +193,11 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
           if not(condition[idx] in conditions):
             conditions.append(condition[idx])
         else:
-          dfReloadedVid.drop([idx2 for idx2, belongsToWell in enumerate(indForWellId) if belongsToWell])
+          dfReloadedVid = dfReloadedVid.drop([idx2 for idx2, belongsToWell in enumerate(indForWellId) if belongsToWell])
+          if 'level_0' in dfReloadedVid.columns:
+            dfReloadedVid = dfReloadedVid.drop(['level_0'], axis=1)
+          dfReloadedVid = dfReloadedVid.reset_index()
+          
       dfParam = pd.concat([dfParam, dfReloadedVid])
     
     # Going through each well of the video
@@ -206,7 +228,7 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
                 # Calculates the global kinematic parameters and stores them the dataframe
                 
                 previousBoutEnd = supstruct["wellPoissMouv"][Well_ID][fishId][NumBout-1]["BoutEnd"] if NumBout > 0 else 0
-                listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, globParam, firstFrame, lastFrame)
+                listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, globParam, firstFrame, lastFrame, minimumFrameToFrameDistanceToBeConsideredAsMoving)
                 
                 toPutInDataFrameColumn = toPutInDataFrameColumn + globParam
                 toPutInDataFrame       = toPutInDataFrame       + listOfGlobalParameters
@@ -254,7 +276,6 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
                   toPutInDataFrame       = toPutInDataFrame       + [tailLength, tailLengthFromRecalculatedAngles] + tailAnglesRecalculatedData + tailAnglesRecalculatedData2.tolist()
                 
                 # Adding bout parameters to the dataframe created for the current well
-                
                 dfParamForWell.loc[curBoutId, toPutInDataFrameColumn] = toPutInDataFrame
                 curBoutId = curBoutId + 1
               
@@ -275,9 +296,9 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
                   # Calculating the global kinematic parameters and more and stores them the dataframe
                   
                   previousBoutEnd = supstruct["wellPoissMouv"][Well_ID][fishId][NumBout-1]["BoutEnd"] if NumBout > 0 else 0
-                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['BoutDuration', 'TotalDistance', 'Speed', 'IBI'], firstFrame, lastFrame)
+                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['BoutDuration', 'TotalDistance', 'Speed', 'IBI'] + addToGlobalParameters, firstFrame, lastFrame, minimumFrameToFrameDistanceToBeConsideredAsMoving)
                   
-                  toPutInDataFrameColumn = toPutInDataFrameColumn + ['BoutDuration', 'TotalDistance', 'Speed', 'IBI']
+                  toPutInDataFrameColumn = toPutInDataFrameColumn + ['BoutDuration', 'TotalDistance', 'Speed', 'IBI'] + addToGlobalParameters
                   toPutInDataFrame       = toPutInDataFrame       + listOfGlobalParameters
                   
                 # Adding bout parameters to the dataframe created for the current well
@@ -298,10 +319,6 @@ def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecrea
   pickle.dump(dfParam,outfile)
   outfile.close()
   
-  file1 = open(os.path.join(resFolder, 'frameStepForDistanceCalculationUsed.json'), "w")
-  file1.write(str(frameStepForDistanceCalculation))
-  file1.close()
-  
   # Saving dataframe for the whole set of videos as a matlab file
   if saveAllBoutsSuperStructuresInMatlabFormat:
     scipy.io.savemat(os.path.join(resFolder, nameOfFile + '.mat'), {'struct1':dfParam.to_dict("list")})
